#! /usr/bin/env python3

import logging
from typing import Optional, List
import httpx
import typer

from snyk_tags import __app_name__, __version__


logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )


def get_org_ids(token: str, group_id: str) -> list:
    """
    Get a list of org_ids based on the given group_id
    :param token:
    :param group_id:
    :return: list of org_ids
    """
    org_ids = []

    with create_client(token=token) as client:
        req = client.get(f"group/{group_id}/orgs")
        if req.status_code == 404:
            logging.error(
            f"Group id: {group_id} is invalid. Error message: {req.json()}."
        )
        orgs = client.get(f"group/{group_id}/orgs").json()
        
        for org in orgs.get("orgs"):
            org_ids.append(org["id"])
    return org_ids

# Apply tags
def apply_tag_to_project(
    client: httpx.Client, org_id: str, project_id: str, tag: str, key: str
) -> tuple:
    """
    Apply the Container tag to each given project

    :param client:
    :param org_id:
    :param project_id:
    :return: tuple of the status_code and dictionary of the JSON response
    """
    tag_data = {
        "key": key,
        "value": tag,
    }
    req = client.post(f"org/{org_id}/project/{project_id}/tags", data=tag_data)

    if req.status_code == 200:
        logging.info(f"Successfully added {tag} tags to Project ID: {project_id}.")

    if req.status_code == 422:
        logging.warning(f"{tag} tag is already applied for Project ID: {project_id}.")

    if req.status_code == 404:
        logging.error(
            f"Project not found, likely a READ-ONLY project. Project ID: {project_id}. Error message: {req.json()}."
        )

    return req.status_code, req.json()


def apply_tags_to_projects(token: str, org_ids: list, type: str, tag: str, key: str) -> None:
    """
    Apply the tags to all Container projects within all orgs given in a list
    :param token:
    :param org_ids:
    :return: None
    """
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            for project in projects.get("projects"):
                if project["type"] == type:
                    logging.debug(
                        apply_tag_to_project(
                            client=client, org_id=org_id, project_id=project["id"], tag=tag, key=key
                        )
                    )

# SAST Command
@app.command(help="Apply SAST tag to Snyk Code files")
def sast(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID if you only want to apply to one org"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"])
    ):

    logging.info(
        "This script will add the SAST tag to every Snyk Code project in Snyk for easy filtering via the UI"
    )
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, type='sast', tag='SAST', key='Type')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, type='sast', tag='SAST', key='Type')

# IaC Command
@app.command(help="Apply IaC tag to Snyk IaC files")
def iac(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID if you only want to apply to one org"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"])
    ):

    logging.info(
        "This script will add the IaC tag to every Snyk IaC project in Snyk for easy filtering via the UI"
    )
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, type='iac', tag='IaC', key='Type')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, type='iac', tag='IaC', key='Type')

# SCA Command
@app.command(help="Apply SCA tag to the preferred project type (default: maven)")
def sca(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand,
            envvar=["ORG_ID"],
            help="Specify one Organization ID if you only want to apply to one org"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  scaType: str = typer.Option(
            "maven", # Default value of comamand
            help="Type of package to update tags: maven, npm"
        )
    ):

    logging.info(
        "This script will add the SCA tag to every Snyk Open Source project in Snyk for easy filtering via the UI"
    )
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, scaType, tag='SCA', key='Type')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, scaType, tag='SCA', key='Type')

# Container Command
@app.command(help="Apply Container tag to the preferred project type (default: deb)")
def container(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID if you only want to apply to one org"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  containerType: str = typer.Option(
            "deb", # Default value of comamand
            help="Type of container to update tags: Dockerfile, deb"
        )
    ):

    logging.info(
        "This script will add the Container tag to every Snyk Container project in Snyk for easy filtering via the UI"
    )
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, containerType, tag='Container', key='Type')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, containerType, tag='Container', key='Type')

# Custom Command
@app.command(help="Apply custom tags to the preferred project type")
def custom(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID if you only want to apply to one org"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  projectType: str = typer.Option(
            ..., # Default value of comamand
            help="Type of project to update value to: sast, iac, deb, Dockerfile, maven, npm..."
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):

    logging.info(
        "This script will add the Container tag to every Snyk Container project in Snyk for easy filtering via the UI"
    )
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, projectType, tagValue, tagKey)
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, projectType, tagValue, tagKey)






#! /usr/bin/env python3

import logging
import typer

from typing import Optional
from snyk_tags import __app_name__, __version__, apply


logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer(help="Use snyk-tags to apply tagging to a set of Snyk projects based on the project type")
app.add_typer(apply.app, name="apply", help="Apply tags to the different project types")

def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"snyk-tags v{__version__}")
        raise typer.Exit()

@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snyk_tags']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.5,<0.5.0',
 'httpx>=0.20.0,<0.21.0',
 'shellingham>=1.4.0,<2.0.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['snyk-tags = snyk_tags.tags:app']}

setup_kwargs = {
    'name': 'snyk-tags',
    'version': '0.2.1',
    'description': 'Tool designed to add tags in bulk to Snyk projects',
    'long_description': '# Snyk Tags Tool\n\nSnyk Tags is a CLI tool that uses the Snyk Project API to assign tags in bulk to Snyk projects based on the type.\n\nSnyk Tags will update all projects of a type within a specific Snyk Group with either an SCA, SAST, IaC or Container tag to help filter projects by Snyk product.\n\nYou can also specify applying the tags to a specific Snyk organization and create your own custom tags.\n\nOnce this is run, go into the UI and click on the tags filter in the projects page (left-hand menu). Select the Type tag and the product as the key. All your Snyk projects from a specific product will be shown via this filter.\n\n## Installation and requirements\n\n### Requirements\n\nRequires Python version above 3.6\n\n### Installation\n\nTo install the simplest way is to use pip:\n\n```bash\npip install snyk-tags\n```\n\nAlternatively you can clone the repo and then run the following commands:\n\n```python\npoetry install # To install dependencies\npython -m snyk-tags # To run snyk-tags\n```\n\n## Usage\n\n**Usage:** snyk-tags [OPTIONS] COMMAND [ARGS]\n\n**COMMAND**:\n\n- apply: ```snyk-tags apply --help```\n  - container: ```snyk-tags apply container```\n  - iac: ```snyk-tags apply iac```\n  - sast: ```snyk-tags apply sast```\n  - sca: ```snyk-tags apply sca```\n  - custom: ```snyk-tags apply custom```\n\n**OPTIONS**:\n\n- **[-v, --version]**: ```snyk tags -v```\n- **[--containertype]**: ```snyk-tags apply container --containertype=deb```\n  - Define the type of Snyk Container projects to tag\n- **[--scatype]**: ```snyk-tags apply sca --scatype=maven```\n  - Define the type of Snyk Open Source projects to tag\n- **[--projecttype]**: ```snyk-tags apply custom --projecttype=maven --tagkey=Type --tagvalue=Value```\n  - Define the type of project to tag, must be accompanied by ```tagkey``` and ```tagvalue```\n- **[--tagkey]**: ```snyk-tags apply custom --projecttype=deb --tagkey=Type --tagvalue=Value```\n  - Define the custom tag\n- **[--tagvalue]**: ```snyk-tags apply custom --projecttype=iac --tagkey=Type --tagvalue=Value```\n  - Define the value of the custom tag\n\n**ARGS**:\n\n- **[--group-id]**: ```snyk tags sast --group-id```\n  - Define the Group ID you want to apply the tags to\n  - Can also be imported as an environment variable\n- **[--org-id]**: ```snyk tags sast --group-id```\n  - Define the Organization ID you want to apply the tags to\n  - Can also be imported as an environment variable\n- **[--token]**: ```snyk-tags apply sast --token=xxx```\n  - Define the Snyk API Token you want to use (needs Group access by default)\n  - Can also be imported as an environment variable\n',
    'author': 'EricFernandezSnyk',
    'author_email': 'eric.fernandez@snyk.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/EricFernandezSnyk/snyk-tags-tool',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
